package tr.edu.bilkent.cs.partitioning.ccp;

/**
 * Partition interface.
 * This interface represents a partition of a 1D workload array.
 *
 * Date: 14.08.2005
 */
public interface Partition {
    /**
     * The separator is set location m for processor pi. i.e. s_p = m
     *
     * @param pi the processor
     * @param m the separator index.
     */
    void setSeparator(int pi, int m);

    /**
     * the underlying 1D separator array.
     * @return seps.
     */
    int[] getSeps();

    /**
     * partition bottleneck value.
     * @return partition value.
     */
    double getPartitionValue();

    /**
     * the prefix-summed task array.
     * @param psum task array.
     */
    void setArray(PrefixSum psum);

    /**
     * the separator value for processor pi.
     *
     * @param pi the processor.
     * @return separator value.
     */
    int getSeparator(int pi);

    /**
     * returns the bottleneck processor.
     * @return bottleneck processor.
     */
    int getBottleneckP();
}
