package tr.edu.bilkent.cs.partitioning.ccp;

/**
 * A Partition implementation for homogenous ccp.
 * <p>
 * 14.08.2005
 * Time: 20:23:22
 */
public class PartitionImpl implements Partition {
    /**
     * the underlying separator array.
     */
    private int[] seps;

    /**
     * The been-there-done-that array: A separator value cannot be set again to be safe.
     */
    private boolean[] btdt;

    /**
     * The underlying prefix-summed task array.
     */
    private PrefixSum psum;

    /**
     * The constructor.
     *
     * @param p the number of processors.
     * @param n the number of tasks.
     */
    public PartitionImpl(int p, int n) {
        if (p <= 0)
            throw new IllegalArgumentException("p must be positive");
        if (n <= 0)
            throw new IllegalArgumentException("n must be positive");

        seps = new int[p];
        btdt = new boolean[p];
        setSeparator(p - 1, n - 1);
    }

    public void setSeparator(int pi, int m) {
        btdt[pi] = true;
        seps[pi] = m;
    }

    public int[] getSeps() {
        return seps;
    }

    public double getPartitionValue() {
        int pre = -1;
        double max = -Double.MAX_VALUE;
        int n = seps.length;
        for (int j = 0; j < n; j++) {
            int i = seps[j];
            double v = getNeckValue(i, pre, j);
            if (v > max)
                max = v;
            pre = i;
        }
        return max;
    }

    /**
     * The load of processor p, if it gets the tasks (pre, i].
     *
     * @param i   the index of last processor to be assigned.
     * @param pre the index of the task prior to the first processor to be assigned.
     * @param p   the processor index. (inapplicable for in homogenous systems)
     * @return the load of processor p.
     */
    protected double getNeckValue(int i, int pre, int p) {
        return psum.getSum(i, pre);
    }

    public void setArray(PrefixSum psum) {
        this.psum = psum;
    }

    public int getSeparator(int pi) {
        if (pi == -1)
            return -1;
        return seps[pi];
    }

    public int getBottleneckP() {
        int pre = -1;
        double max = -Double.MAX_VALUE;
        int maxJ = -1;
        int n = seps.length;
        for (int j = 0; j < n; j++) {
            int i = seps[j];
            double v = getNeckValue(i, pre, j);

            if (v > max) {
                max = v;
                maxJ = j;
            }
            pre = i;
        }
        return maxJ;
    }


    @Override
    public String toString() {
        int pre = 0;
        StringBuilder buf = new StringBuilder(10);
        buf.append(this.getPartitionValue());
        buf.append(' ');
        buf.append(this.getBottleneckP());
        buf.append(" [");
        for (int i = 0; i < seps.length; i++) {
            if (i != 0) {
                buf.append(" | ");
            }
            int sep = seps[i];
            for (int j = pre; j <= sep; j++) {
                buf.append(psum.getAt(j));
                buf.append(' ');
            }
            pre = sep + 1;
        }
        buf.append(']');
        return buf.toString();
    }
}
