package tr.edu.bilkent.cs.partitioning.ccp;

import java.util.Arrays;

/**
 * A default PrefixSum implementation.
 *
 * Date: 14.08.2005
 */
public class PrefixSumImpl implements PrefixSum {
    /**
     * The prefix-summed array.
     */
    private double[] sum;

    /**
     * The ctor.
     * Note that, the parameter is used without being cloned.
     *
     * @param sum the prefix-summed array.
     */
    public PrefixSumImpl(double[] sum) {
        this.sum = sum;
    }

    public int size() {
        return sum.length;
    }

    public double get(int i) {
        if (i == -1)
            return 0;
        return sum[i];
    }

    public double getSum(int i, int j) {
        if (j == -1) {
            return get(i);
        }
        return get(i) - get(j);
    }

    public double getSum() {
        return get(size() - 1);
    }

    public int search(double v) {
        int k = Arrays.binarySearch(sum, v);
        if (k >= 0)
            return k;
        k = -k - 1;
        if (k == 0)
            return 0;
        if (k >= sum.length)
            return k - 1;
        double p = sum[k - 1];
        double q = sum[k];
        if (q - v < v - p)
            return k;
        return k - 1;
    }

    public int searchLessThan(double v) {
        int a = Arrays.binarySearch(sum, v);
        if (a >= 0)
            return a;
        return -a - 2;
    }

    public int searchGreaterThan(double v) {
        int a = Arrays.binarySearch(sum, v);
        if (a >= 0)
            return a;
        return -a - 1;
    }

    public double getAt(int p) {
        if (p == 0)
            return get(p);
        return get(p) - get(p - 1);
    }

    public double[] getArray() {
        return sum;
    }

    public double[] getOriginalValues(){
        double[] e = sum.clone();
        for (int i=e.length-1; i>0; i--){
            e[i] -= e[i-1];
        }
        return e;
    }

    @Override public String toString() {
        StringBuilder sb = new StringBuilder(10);
        sb.append('[');
        for (int i = 0; i < size(); i++) {
            if (i > 0)
                sb.append(", ");

            sb.append(getAt(i));
        }
        sb.append(']');
        return sb.toString();
    }
}
