package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;
import tr.edu.bilkent.cs.partitioning.ccp.Partition;

/**
 * AbstractHeterogeneousPartitionMethod. The abstract implementation of HeterogeneousPartitionMethod
 *
 * Date: 03.02.2006
 */
public abstract class AbstractHeterogeneousPartitionMethod implements HeterogeneousPartitionMethod {
    /**
     * calculates and returns the partition.
     * @param psArray the prefix-summed task array.
     * @param processors the prefix-summed processor array.
     * @param wmax the max of the task array.
     * @param emin the min of the processor array.
     * @return the calculated partition.
     */
    public Partition calculatePartition(PrefixSum psArray, PrefixSum processors, double wmax, double emin) {
        double bStar = findBStar(psArray, processors, wmax, emin);
        return getPartitionFor(psArray, processors, bStar);
    }

    /**
     * Returns the partition for the computed bottleneck value.
     *
     * @param psArray the prefix-summed task array.
     * @param processors the prefix-summed processor array.
     * @param bStar the bottleneck value.
     * @return the calculated partition.
     */
    @SuppressWarnings({"FeatureEnvy"})
    protected static Partition getPartitionFor(PrefixSum psArray, PrefixSum processors, double bStar) {
        int n = psArray.size();
        int p = processors.size();
        Partition partition = new HetPartitionImpl(processors, n);

        partition.setArray(psArray);
        double sum = 0;
        for (int k = 0; k < p - 1; k++) {
            double e_k = processors.getAt(k);
            double b = bStar * e_k;
            double sumb = sum + b;
            int m = psArray.searchLessThan(sumb);
            sum = psArray.get(m);
            partition.setSeparator(k, m);
        }
        return partition;
    }

    /**
     * Calculates and returns the bottleneck value.
     *
     * @param psArray the prefix-summed task array.
     * @param processors the prefix-summed processor array.
     * @param wmax the max of the task array.
     * @param emin the min of the processor array.
     * @return the bottleneck value. 
     */
    protected abstract double findBStar(PrefixSum psArray, PrefixSum processors, double wmax, double emin);
}
