package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

/**
 * Bound interface. Represents a [LB, UB] or (LB, UB] bound.
 *
 * Date: 20.04.2006
 */
public interface Bound {
    /**
     * compares the given value with the range.
     *
     * @param candidateB the value to be compared.
     * @return a BoundCompareResult representing the comparison.
     */
    BoundCompareResult compare(double candidateB);

    /**
     * returns the lower bound.
     * @return lb.
     */
    double getLb();

    /**
     * sets lower bound. if lbEpsilon is true, than indicates that LB value is exclusive to the range.
     * @param lb lower bound.
     * @param lbEpsilon is LB exclusive to the range.
     */
    void setLb(double lb, boolean lbEpsilon);

    /**
     * returns upper bound.
     * @return upper bound.
     */
    double getUb();

    /**
     * sets upper bound.
     * @param ub upper bound.
     */
    void setUb(double ub);

    /**
     * returns mid point, i.e., (LB+UB)/2.
     * @return mid point.
     */
    double getMid();

    /**
     * return diff, i.e. (UB-LB).
     * @return range length.
     */
    double getDiff();
}
