package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

/**
 * A Bound implementation.
 *
 * Date: 15.02.2006
 */
public class BoundImpl implements Bound {
    /**
     * two doubles are considered equal if they are this close to each other.
     */
    private static final double EPSILON = Double.MIN_VALUE;

    /**
     * represents the lower bound.
     */
    private double lb;

    /**
     * represents the upper bound.
     */
    private double ub;

    /**
     * is lb exclusive to the range.
     */
    private boolean lbEpsilon;

    /**
     * The constructor.
     *
     * @param lb lower bound, this lb is inclusive.
     * @param ub upper bound.
     */
    public BoundImpl(double lb, double ub) {
        setLb(lb, false);
        setUb(ub);
    }

    public BoundCompareResult compare(double candidateB) {
        double lowerBound = getLb();
        if (candidateB < lowerBound)
            return BoundCompareResult.LESS;
        if (Math.abs(candidateB - lowerBound)<=EPSILON){
            if (lbEpsilon)
                return BoundCompareResult.LESS;
            return BoundCompareResult.LB;
        }
        double upperBound = getUb();
        if (candidateB < upperBound)
            return BoundCompareResult.BETWEEN;
        if (Math.abs(candidateB - upperBound)<=EPSILON)
            return BoundCompareResult.UB;
        return BoundCompareResult.GREATER;
    }

    public double getLb() {
        return lb;
    }

    public final void setLb(double lb, boolean lbEpsilon) {
        this.lb = lb;
        this.lbEpsilon = lbEpsilon;
    }

    public double getUb() {
        return ub;
    }

    public final void setUb(double ub) {
        this.ub = ub;
    }

    public double getMid() {
        return (ub + lb) /2;
    }

    public double getDiff() {
        return ub - lb;
    }

    @Override
    public String toString() {
        String leftpar;
        if (lbEpsilon)
            leftpar = "(";
        else
            leftpar = "[";
        return leftpar + lb + ", " + ub + ']';
    }
}
