package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;

/**
 * An interval crobber. Used especially in Nicol related implementations.
 *
 * Date: 13.02.2006
 */
public interface Crobber{
    /**
     * crops a given interval.
     *
     * @param psArray prefix-summed task array.
     * @param processors prefix-summed processor array.
     * @param candidateB candidate bottleneck value.
     * @param mid index for the mid of interval.
     * @param interval the interval to be crobbed.
     * @param p the index of the processor.
     * @return whether the interval can be crobbed or not.
     */
    boolean cropInterval(PrefixSum psArray, PrefixSum processors, double candidateB, int mid, Interval interval, int p);

    /**
     * constructs an interval for the given parameters.
     *
     * @param first the lb of the interval.
     * @param last the ub of the interval.
     * @param p the processor index.
     * @return constructed interval.
     */
    Interval getInterval(int first, int last, int p);

    /**
     * A candidate bottleneck value.
     * In some situations, a candidate value may not be proved by a probe, and it may be required to perform one more probe for that candidate value.
     *
     * @param psArray prefix-summed task array.
     * @param processors prefix-summed processor array.
     * @param candidate candidate bottleneck value.
     * @param p processor index.
     * @return the not-probed candidate. 
     */
    double getNotProbedCandidate(PrefixSum psArray, PrefixSum processors, double candidate, int p);
}
