package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;

/**
 * Heterogeneous DP+ implementation.
 * The algorithm is described in Section 4.2 of:
 * A. Pinar, K. Tabak, C. Aykanat, One-Dimensional Partitioning for Heterogeneous Systems: Theory and Practice,
 * J. of Parallel and Distributed Computing, 2008.
 */
public class DPPlus extends AbstractHeterogeneousPartitionMethod {
    private static double getBp(double[] bp, int j) {
        if (j == -1)
            return 0;
        return bp[j];
    }

    @Override
    protected double findBStar(PrefixSum psArray, PrefixSum processors, double wmax, double emin) {
        SeparatorBounds separatorBounds = SeparatorBoundsGenerator.getSeparatorBounds(psArray, processors);

        double[] E = processors.getArray().clone();
        for (int i = E.length - 1; i > 0; i--) {
            E[i] -= E[i - 1];
        }

        double[] B_p_1 = psArray.getArray().clone();

        int firstLow = separatorBounds.getLow(0);
        int firstHigh = separatorBounds.getHigh(0);
        for (int i = firstLow; i <= firstHigh; i++) {
            B_p_1[i] /= E[0];
        }

        double[] B_p = B_p_1.clone();

        int P = E.length;
        int N = B_p_1.length;

        for (int p = 1; p < P; p++) {
            int pLow = separatorBounds.getLow(p);
            int pHigh = separatorBounds.getHigh(p);
            double[] tmp = B_p_1;
            B_p_1 = B_p;
            B_p = tmp;
            int j = separatorBounds.getLow(p - 1);
            for (int i = pLow; i <= pHigh; i++) {
                if ((psArray.getSum(i, j) / E[p]) <= getBp(B_p_1, j)) {
                    B_p[i] = getBp(B_p_1, j);
                } else {
                    double sum;
                    do {
                        j++;
                        sum = psArray.getSum(i, j);
                    } while ((sum / E[p]) > getBp(B_p_1, j) && j < i);

                    {
                        double prebp = getBp(B_p_1, j);
                        double lastbp = sum / E[p];
                        if (lastbp == prebp) {
                            B_p[i] = prebp;
                        } else if (lastbp > prebp) {
                            throw new RuntimeException("not possible");
                        } else if (psArray.getSum(i, j - 1) / E[p] < prebp) {
                            j--;
                            B_p[i] = psArray.getSum(i, j) / E[p];
                        } else {
                            B_p[i] = prebp;
                        }

                    }
                }
            }
        }
        return B_p[N - 1];
    }

    public String getName() {
        return "DP+";
    }
}
