package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.Partition;
import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;

/**
 * Heterogeneous Bisection algorithm as described in:
 * A. Pinar, K. Tabak, C. Aykanat, One-Dimensional Partitioning for Heterogeneous Systems: Theory and Practice,
 * J. of Parallel and Distributed Computing, 2008.
 */
public class HeterogeneousBisection extends AbstractHeterogeneousPartitionMethod {
    private static final Probe1D prober = Probe1DFactory.getProbe1DImpl();
    private double epsilon;

    public HeterogeneousBisection(double epsilon) {
        this.epsilon = epsilon;
    }

    @SuppressWarnings({"FeatureEnvy"})
    public static Bound calculateBound(PrefixSum psArray, PrefixSum processors, double epsilon) {
        HeterogeneousPartitionMethod h3 = new RB();
        Partition partition = h3.calculatePartition(psArray, processors, 0, 0);
        double ub = partition.getPartitionValue();
        double lb = calculateInitialLb(psArray, processors);
        double bepsilon = (ub - lb) * epsilon;
        Bound bound = new BoundImpl(lb, ub);
        while (bound.getDiff() >= bepsilon) {
            double mid = bound.getMid();
            if (prober.probe(psArray, processors, mid))
                bound.setUb(mid);
            else
                bound.setLb(mid, true);
        }
        return bound;
    }

    private static double calculateInitialLb(PrefixSum psArray, PrefixSum processors) {
        return psArray.getSum() / processors.getSum();
    }

    @Override
    protected double findBStar(PrefixSum psArray, PrefixSum processors, double wmax, double emin) {
        Bound bound = calculateBound(psArray, processors, epsilon);
        return bound.getUb();
    }

    public String getName() {
        return "e-BS[" + epsilon + ']';
    }
}
