package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;
import tr.edu.bilkent.cs.partitioning.ccp.Partition;

/**
 * The interface for Heterogeneous Partition Method.
 * All heterogeneous partitioning algorithms shall implement that interface.
 *
 * Date: 30.03.2006
 */
public interface HeterogeneousPartitionMethod {
    /**
     * partition calculate. The main method of the algorithm.
     *
     * @param psArray prefix-summed Workload Array
     * @param processors prefix-summed processor powers.
     * @param wmax maximum load
     * @param emin minimum processor power
     *
     * @return the partition.
     */
    Partition calculatePartition(PrefixSum psArray, PrefixSum processors, double wmax, double emin);

    /**
     * An identifying string for the algorithm.
     *
     * @return the name of the algorithm.
     */
    String getName();
}
