package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.Partition;
import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;

/**
 * LRProbe implementation as described in:
 * A. Pinar, K. Tabak, C. Aykanat, One-Dimensional Partitioning for Heterogeneous Systems: Theory and Practice,
 * J. of Parallel and Distributed Computing, 2008.
 */
public class LRProbeImpl implements LRProbe {
    /**
     * counter for probe counts.
     */
    private Counter counter;

    /**
     * Constructor
     *
     * @param counter counter
     */
    public LRProbeImpl(Counter counter) {
        this.counter = counter;
    }

    public ProbeResult probe(PrefixSum workloadPs, PrefixSum processors, double candidate) {
        counter.incrementCount();
        int p = processors.size();
        int sn = workloadPs.size();
        double sum = 0;

        Partition partition = new HetPartitionImpl(processors, sn);
        partition.setArray(workloadPs);

        for (int k = 0; k < p - 1; k++) {
            double e_k = processors.getAt(k);
            double b = candidate * e_k;
            double sumb = sum + b;
            int m = workloadPs.searchLessThan(sumb);
            sum = workloadPs.get(m);
            partition.setSeparator(k, m);
        }
        boolean probed = false;

        sum += candidate * processors.getAt(p - 1); // the loop above did not count the last processor
        if (sum >= workloadPs.getSum())
            probed = true;
        return new ProbeResultImpl(probed, partition);
    }
}
