package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;
import tr.edu.bilkent.cs.partitioning.ccp.Partition;

/**
 * The generalization of the Miguet and Pierson's heuristic.
 * The generalization algorithm is presented in:
 * A. Pinar, K. Tabak, C. Aykanat, One-Dimensional Partitioning for Heterogeneous Systems: Theory and Practice,
 * J. of Parallel and Distributed Computing, 2008.
 *
 * The Miguet and Pierson's original heuristic is presented in:
 * S.Miguet, J.-M. Pierson, Heuristics for 1D rectilinear partitioning as a low cost and high quality
 * answer to dynamic load balancing, in: HPCN Europe 97: Proceedings of the International
 * Conference and Exhibition on High-Performance Computing and Networking, Springer-Verlag, London, UK, 1997.
 */
public class MP implements HeterogeneousPartitionMethod {
    private static final String NAME = "MP";

    public String getName(){
        return NAME;
    }

    public Partition calculatePartition(PrefixSum psArray, PrefixSum processors, double wmax, double emin) {
        int n = psArray.size();      // N
        int p = processors.size();   // P
        Partition partition = new HetPartitionImpl(processors, n);

        double v = psArray.get(n - 1); // W_{1,N}
        double pt = processors.get(p-1); // E_{1,P}
        for (int k = 0; k < p - 1; k++) {
            double pk = processors.get(k); // E_{1,p}
            double w = v * pk / pt;  // the ratio
            int m = psArray.search(w);   // s_p
            partition.setSeparator(k, m);
        }

        partition.setArray(psArray);

        return partition;
    }

}
