package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;

/**
 * Heterogeneous Nicol algorithm as described in Section 4.3.1 of:
 * A. Pinar, K. Tabak, C. Aykanat, One-Dimensional Partitioning for Heterogeneous Systems: Theory and Practice,
 * J. of Parallel and Distributed Computing, 2008.
 */
public class Nicol extends NicolMinus {

    @Override
    protected Crobber createCrobber(PrefixSum psArray, PrefixSum processors) {
        double totalSum = psArray.getSum();
        double lb = totalSum / processors.getSum();
        double ub = new MP().calculatePartition(psArray, processors, 0, 0).getPartitionValue();
        return new NicolCrobber(new BoundImpl(lb, ub));
    }

    @Override
    public String getName() {
        return "Het_Nicol";
    }

    public class NicolCrobber implements Crobber {
        private Bound bound;

        public NicolCrobber(Bound bound) {
            this.bound = bound;
        }

        public boolean cropInterval(PrefixSum psArray, PrefixSum processors, double candidateB, int mid, Interval interval, int p) {
            BoundCompareResult comp = bound.compare(candidateB);
            if (comp == BoundCompareResult.LESS) {
                interval.setFirst(mid + 1);
                return true;
            }
            if (comp == BoundCompareResult.UB || comp == BoundCompareResult.GREATER) {
                interval.setLast(mid);
                return true;
            }
            if (getProber().probe(psArray, processors, candidateB)) {
                interval.setLast(mid);
                bound.setUb(candidateB);
                return true;
            }
            interval.setFirst(mid + 1);
            bound.setLb(candidateB, true);
            return true;
        }

        public Interval getInterval(int first, int last, int p) {
            Interval interval = new IntervalImpl();
            interval.setFirst(first);
            interval.setLast(last);
            return interval;
        }

        public double getNotProbedCandidate(PrefixSum psArray, PrefixSum processors, double candidate, int p) {
            return candidate;
        }
    }
}
