package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;

/**
 * This is the very original Nicol algorithm.
 */
public class NicolMinus extends AbstractHeterogeneousPartitionMethod {
    /**
     * The one-dimensional prober.
     */
    private Probe1D prober = Probe1DFactory.getProbe1DImpl();

    private static int searchFirstProbe(int first, int last, PrefixSum psArray, PrefixSum processors, int p, Crobber crobber) {
        double ep = processors.getAt(p);
        Interval interval = crobber.getInterval(first, last, p);
        boolean probedOnce = false;
        while (!interval.converged()) {
            int mid = interval.getMidPoint();
            double candidateB = psArray.getSum(mid, first - 1) / ep;
            if (crobber.cropInterval(psArray, processors, candidateB, mid, interval, p))
                probedOnce = true;
        }
        if (probedOnce)
            return interval.getLast();
        return -interval.getLast() - 1;
    }

    @Override
    protected double findBStar(PrefixSum psArray, PrefixSum processors, double wmax, double emin) {
        // teker teker processor'leri bottleneck yapiyoruz.
        // sonuncuyu denemeye gerek yok.
        double bottleneckValue = Double.MAX_VALUE;
        Crobber crobber = createCrobber(psArray, processors);
        int last = 0;
        int p = processors.size();
        int sn = psArray.size();
        for (int b = 0; b < p - 1; b++) {
            double ep = processors.getAt(b);
            int next = searchFirstProbe(last, sn - 1, psArray, processors, b, crobber);
            double candidate;
            if (next >= 0) {
                candidate = psArray.getSum(next, last - 1) / ep;
//                System.out.println(b+" " + candidate);
            } else {
                next = -next - 1;
                candidate = psArray.getSum(next, last - 1) / ep;
                if (candidate < bottleneckValue)
                    candidate = crobber.getNotProbedCandidate(psArray, processors, candidate, b);
                if (candidate == Double.MAX_VALUE)
                    next++;
            }
            if (candidate < bottleneckValue) {
                bottleneckValue = candidate;
            }
            last = next;
        }
        double candidate = psArray.getSum(sn - 1, last - 1) / processors.getAt(p - 1);
        if (candidate < bottleneckValue)
            return candidate;
        return bottleneckValue;
    }

    protected Crobber createCrobber(PrefixSum psArray, PrefixSum processors) {
        return new DefaultCrobber();
    }

    protected Probe1D getProber() {
        return prober;
    }

    public String getName() {
        return "Het_Nicol-";
    }

    private class DefaultCrobber implements Crobber {
        public boolean cropInterval(PrefixSum psArray, PrefixSum processors, double candidateB, int mid, Interval interval, int p) {
            if (prober.probe(psArray, processors, candidateB))
                interval.setLast(mid);
            else
                interval.setFirst(mid + 1);
            return true;
        }

        public Interval getInterval(int first, int last, int p) {
            Interval interval = new IntervalImpl();
            interval.setFirst(first);
            interval.setLast(last);
            return interval;
        }

        public double getNotProbedCandidate(PrefixSum psArray, PrefixSum processors, double candidate, int p) {
            throw new UnsupportedOperationException("not yet implemented");
        }
    }
}
