package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;

/**
 * One-dimensional probe interface.
 * Interface defines a contract for the probe function.
 * <p>
 * The probe function requirements and definition, as well as an implementation is presented in:
 * A. Pinar, K. Tabak, C. Aykanat, One-Dimensional Partitioning for Heterogeneous Systems: Theory and Practice,
 * J. of Parallel and Distributed Computing, 2008.
 */
public interface Probe1D {
    /**
     * returns whether candidate probe value can be satisfied for the workloads (workloadPs) on the processors
     * (processors)
     *
     * @param workloadPs prefix-summed workload array
     * @param processors prefix-summed processor array
     * @param candidate candidate bottleneck value to evaluate
     * @return whether candidate can be successfully probed
     */
    boolean probe(PrefixSum workloadPs, PrefixSum processors, double candidate);
}
