package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.Partition;
import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;

/**
 * RecursiveBisection method, as described in the {One-Dimensional Partitioning for Heterogeneous Systems} by A. Pinar, et.al.
 *
 */
public class RB implements HeterogeneousPartitionMethod {

    @Override
    public String getName() {
        return "RB";
    }

    @Override
    public Partition calculatePartition(PrefixSum psArray, PrefixSum processors, double wmax, double emin) {
        int n = psArray.size();
        int P = processors.size();
        Partition partition = new HetPartitionImpl(processors, n);
        partition.setArray(psArray);

        bisect(psArray, processors, partition, 0, P - 1);

        return partition;
    }

    protected void bisect(PrefixSum psArray, PrefixSum processors, Partition partition, int p, int r) {
        if (p == r)
            return;

        int preIndex = p - 1;
        int preSep = partition.getSeparator(preIndex); // s_{p-1}
        int s_r = partition.getSeparator(r);

        // getSum() returns the sum (preSep, s_r], i.e. excluding presep, including s_r
        double W_tot = psArray.getSum(s_r, preSep);

        double preW = psArray.get(preSep); // W_{1, s_{p-1}}

        double E_pr = processors.getSum(r, preIndex);

        int q = pickMidPoint(processors, preIndex, E_pr, p, r);
        double E_pq = processors.getSum(q, preIndex);

        double Wfirst = W_tot * E_pq / E_pr;
        double W = Wfirst + preW;
        int s_q = psArray.search(W);
        partition.setSeparator(q, s_q);

        bisect(psArray, processors, partition, p, q);
        bisect(psArray, processors, partition, q + 1, r);
    }

    protected int pickMidPoint(PrefixSum processors, int preIndex, double mySpeed, int p, int r) {
        return (p + r - 1) / 2;
    }
}
