package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.PrefixSum;
import tr.edu.bilkent.cs.partitioning.ccp.Partition;

/**
 * A RL-Probe implementation of the LR-Probe interface.
 */
public class RLProbeImpl implements LRProbe {
    /**
     * Counter for probes.
     */
    private Counter counter;

    /**
     * Constructor.
     *
     * @param counter injected probe counter
     */
    public RLProbeImpl(Counter counter) {
        this.counter = counter;
    }

    public ProbeResult probe(PrefixSum workloadPs, PrefixSum processors, double candidate) {
        counter.incrementCount();
        int p = processors.size();
        int sn = workloadPs.size();
        double sum = workloadPs.getSum();

        Partition partition = new HetPartitionImpl(processors, sn);
        partition.setArray(workloadPs);

        for (int k = p-1; k > 0; k--) {
            double e_k = processors.getAt(k);
            double b = candidate * e_k;
            double sumb = sum - b;
            int m = workloadPs.searchGreaterThan(sumb);
            sum = workloadPs.get(m);
            partition.setSeparator(k-1, m);
        }
        boolean probed = false;

        sum -= candidate*processors.getAt(0);
        if (sum <= 0)
            probed = true;
        return new ProbeResultImpl(probed, partition);
    }
}
