package tr.edu.bilkent.cs.partitioning.ccp.hetccp;

import tr.edu.bilkent.cs.partitioning.ccp.Partition;

/**
 * SeparatorBounds interface.
 * This interface defines a data-class contract for separator bounds of processors.
 * In the contract, each processor has a candidate separator bounds interval.
 */
public interface SeparatorBounds {
    /**
     * returns the lowest possible separator index for processor p.
     *
     * @param p processor index
     *
     * @return the lowest possible separator index for processor p.
     */
    int getLow(int p);

    /**
     * returns the highest possible separator index for processor p.
     *
     * @param p processor index
     *
     * @return the highest possible separator index for processor p.
     */
    int getHigh(int p);

    /**
     * returns the probe value that generated the separator bounds.
     *
     * @return probe value
     */
    double getProbeValue();

    /**
     * This function updates for a successfully probed partition.
     *
     * @param partition successfully probed partition.
     *
     * @param p processor index.
     */
    void updateWithProbedPartition(Partition partition, int p);

    /**
     * This function updates the bounds for a failed probe.
     *
     * @param partition failed probe partition.
     *
     * @param p processor index.
     */
    void updateWithFailedProbe(Partition partition, int p);
}
