package tr.edu.bilkent.cs.partitioning.ccp.hetccp.cp;

/**
 * The interface for running statistics.
 * The interface is designed to keep the minimum memory footprint while observing double entities and returning several
 * statistical measures such as mean and variance.
 */
public interface RunningStatistics {
    /**
     * Adds an observation to the statistics.
     *
     * @param x observation
     */
    void addObservation(double x);

    /**
     * Returns the mean of observations.
     *
     * @return mean
     */
    double getMean();

    /**
     * variance of observations.
     *
     * @return variance.
     */
    double getVariance();

    /**
     * standard deviation.
     *
     * @return standard deviation
     */
    double getStandardDeviation();

    /**
     * coefficient of variance
     *
     * @return coefficient of variance
     */
    double getCoefficientOfVariance();
}
