package tr.edu.bilkent.cs.partitioning.ccp.hetccp.cp;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * RunningStatistics implementation.
 * This implementation is O(1) in add observation, while O(1) in getXXX().
 *
 * Date: Jan 30, 2007
 */
public class RunningStatisticsImpl implements RunningStatistics {
    private BigDecimal sum = new BigDecimal(0);
    private BigDecimal sx2 = new BigDecimal(0); // Sum of [X^2]
    private int n; // number of terms
    private static final int SCALE = 100;

    public void addObservation(double x) {
        sum = sum.add(new BigDecimal(x));
        sx2 = sx2.add(new BigDecimal(x*x));
        n += 1;
    }

    public double getMean(){
        return sum.divide(new BigDecimal(n), SCALE, RoundingMode.HALF_DOWN).doubleValue();
    }

    private double getEx2(){
        return sx2.divide(new BigDecimal(n), SCALE, RoundingMode.HALF_DOWN).doubleValue();
    }

    public double getVariance(){
        double mean = getMean();
        return getEx2() - mean * mean;
    }

    public double getStandardDeviation(){
        return Math.sqrt(getVariance());
    }

    public double getCoefficientOfVariance(){
        return getStandardDeviation()/getMean();
    }
}
